
% Inflate implementation for Haskell

% Copyright 2004 Ian Lynagh <igloo@earth.li>
% Licence: 3 clause BSD.

\section{Inflate}

This module provides a Haskell implementation of the inflate function,
as described by RFC 1951.

\begin{code}
module Data.Compression.Deflate.Inflate (inflate) where

import Control.Monad (unless, liftM)
import Data.Array (Array, array, (//), (!))
import Data.Bits (testBit)
import Data.List (genericTake, genericDrop, genericSplitAt, genericReplicate,
                  mapAccumL, sort)
import Data.Word (Word8, Word16, Word32)
\end{code}

\section{Types}

Type synonyms are your friend.

\begin{code}
type Octet = Word8     -- The inut/output type
type Octets = [Octet]

type Code = Word16     -- A generic code
type Dist = Code       -- A distance code (1-32768)
type LitLen = Code     -- A literal/length code (3-258)
type Length = Word8    -- Number of bits needed to identify a code

type Table = InfM Code -- A Huffman table
type Tables = (Table, Table) -- lit/len and dist Huffman tables
\end{code}

The \verb!Bit! datatype is used for the input. We can show values and
convert to and from octet streams and Word32s.

\begin{code}
newtype Bit = Bit Bool
    deriving Eq
instance Show Bit where
    show = (\x -> [x]) . show_b
    showList bs = showString $ "'" ++ map show_b bs ++ "'"

show_b :: Bit -> Char
show_b (Bit True) = '1'
show_b (Bit False) = '0'

octet_to_bits :: Octet -> [Bit]
octet_to_bits n = map (\i -> Bit (testBit n i)) [0..7]

bits_to_octet :: [Bit] -> Octet
bits_to_octet = foldr (\(Bit b) i -> 2 * i + (if b then 1 else 0)) 0

bits_to_word32 :: [Bit] -> Word32
bits_to_word32 = foldr (\(Bit b) i -> 2 * i + (if b then 1 else 0)) 0

bits_to_word16 :: [Bit] -> Word16
bits_to_word16 = foldr (\(Bit b) i -> 2 * i + (if b then 1 else 0)) 0

bits_to_word8 :: [Bit] -> Word8
bits_to_word8 = foldr (\(Bit b) i -> 2 * i + (if b then 1 else 0)) 0
\end{code}

\section{Monad}

We carry around the input bits remaining, how many we have used mod 8
(so that we can octet-align), the last 32768 output octets in an array
so we can repeat them and where in the (cyclic) array we are. The offset
is rarely used, so we make it strict to avoid building huge closures.

\begin{code}
data State = State { bits :: [Bit],                 -- remaining input bits
                     offset :: !Word8,              -- num bits consumed mod 8
                     history :: Array Word16 Octet, -- last 32768 output words
                     loc :: Word16                  -- where in history we are
                   }
data InfM a = InfM (State -> (a, State))

instance Monad InfM where
 -- (>>=)  :: InfM a -> (a -> InfM b) -> InfM b
    InfM v >>= f = InfM $ \s -> let (x, s') = v s
                                    InfM y = f x
                                in y s'
 -- return :: a -> InfM a
    return x = InfM $ \s -> (x, s)

extract_InfM :: InfM a -> (a, [Bit])
extract_InfM (InfM f) = let (x, s) = f undefined in (x, bits s)

set_bits :: [Bit] -> InfM ()
set_bits bs = InfM $ const ((), State bs 0 (array (0, 32767) []) 0)
\end{code}

We also provide various helper functions to align the remaining bits to
an octet boundary and various functions to get some bits from the input.

\begin{code}
{-
no_bits :: InfM Bool
no_bits = InfM $ \s -> (null (bits s), s)
-}

align_8_bits :: InfM ()
align_8_bits
 = InfM $ \s -> ((), s { bits = genericDrop ((8 - offset s) `mod` 8) (bits s),
                         offset = 0 })

get_bits :: Word8 -> InfM [Bit]
get_bits n = InfM $ \s -> case need n (bits s) of
                              (ys, zs) ->
                                  (ys, s { bits = zs,
                                           offset = (n + offset s) `mod` 8 } )
    where need 0 xs = ([], xs)
          need _ [] = error "get_bits: Don't have enough!"
          need (i+1) (x:xs) = let (ys, zs) = need i xs in (x:ys, zs)

-- b at most 8
-- n at most 65535
get_octets :: Word8 -> Word16 -> InfM Octets
get_octets _ 0 = return []
get_octets b n = do w <- get_octet b
                    ws <- get_octets b (n-1)
                    return (w:ws)

-- i at most 8
get_octet :: Word8 -> InfM Octet
get_octet i = do bs <- get_bits i
                 return (bits_to_octet bs)

-- i at most 16
get_w16 :: Word8 -> InfM Word16
get_w16 i = do bs <- get_bits i
               return (bits_to_word16 bs)

-- i at most 16
get_w8 :: Word8 -> InfM Word8
get_w8 i = do bs <- get_bits i
              return (bits_to_word8 bs)

get_bit :: InfM Bit
get_bit = do [x] <- get_bits 1
             return x
\end{code}

We have 2 ways to provide more output. We can either write a single
octet out or repeat a given number of bits a given distance back in the
history.

\begin{code}
output :: Octet -> InfM ()
output w = InfM $ \s -> let l = loc s
                            in ((), s { history = history s // [(l, w)],
                                        loc = l + 1 })

repeat_w32s :: Word16 -> Word16 -> InfM Octets
repeat_w32s len dist
 = InfM $ \s -> let l = loc s
                    h = history s
                    new = map (h!) $ genericTake dist ([(l - dist) `mod` 32768..32767] ++ [0..])
                    new_bit = genericTake len (cycle new)
                    h' = h // zip (map (`mod` 32768) [l..]) new_bit
                in (new_bit, s { history = h', loc = (l + len) `mod` 32768 })
\end{code}

\section{Inflate itself}

The hardcore stuff!

To inflate an octet stream we use inflate\_blocks to do the hard work.
It in turn looks at the first 3 bits to decide whether to just output an
uncompressed segment or pass off the work to inflate\_tables and
inflate\_codes.

\begin{code}
inflate :: Octets -> (Octets, [Bit])
inflate is = extract_InfM $ do set_bits $ concatMap octet_to_bits is
                               x <- inflate_blocks False
                               align_8_bits
                               return x

-- Bool is true if we have seen the "last" block
inflate_blocks :: Bool -> InfM Octets
inflate_blocks True = return []
inflate_blocks False
     = do [Bit is_last, Bit t1, Bit t2] <- get_bits 3
          case (t1, t2) of
              (False, False) ->
                  do align_8_bits
                     len <- get_w16 16
                     nlen <- get_w16 16
                     -- check nlen = 1s complement of len
                     unless (len + nlen == -1)
                        $ error "inflate_blocks: Mismatched lengths"
                     ws <- get_octets 8 len
                     mapM_ output ws
                     return ws
              (True, False) ->
                  inflate_codes is_last inflate_trees_fixed
              (False, True) ->
                  do tables <- inflate_tables
                     inflate_codes is_last tables
              (True, True) ->
                  error ("inflate_blocks: case 11 reserved")
\end{code}

\begin{code}
inflate_tables :: InfM Tables
inflate_tables
 = do hlit <- get_w16 5
      hdist <- get_w16 5
      hclen <- get_w8 4
      llc_bs <- get_bits ((hclen + 4) * 3)
      let llc_bs' = zip (map bits_to_word8 $ triple llc_bs)
                        [16,17,18,0,8,7,9,6,10,5,11,4,12,3,13,2,14,1,15]
          tab = make_table llc_bs'
      lit_dist_lengths <- make_lit_dist_lengths tab
                                                (258 + hlit + hdist)
                                                (error "inflate_tables dummy")
      let (lit_lengths, dist_lengths) = genericSplitAt (257 + hlit)
                                                       lit_dist_lengths
          lit_table = make_table (zip lit_lengths [0..])
          dist_table = make_table (zip dist_lengths [0..])
      return (lit_table, dist_table)

triple :: [a] -> [[a]]
triple (a:b:c:xs) = [a,b,c]:triple xs
triple [] = []
triple _ = error "triple: can't happen"
\end{code}

make\_lit\_dist\_lengths reads n (at most ~350) dist and length code
lengths.

\begin{code}
make_lit_dist_lengths :: Table -> Word16 -> Length -> InfM [Length]
make_lit_dist_lengths _ n _ | n < 0 = error "make_lit_dist_lengths n < 0"
make_lit_dist_lengths _ 0 _ = return []
make_lit_dist_lengths tab n last_thing
 = do c <- tab
      (ls, n', last_thing') <- meta_code n c last_thing
      ws <- make_lit_dist_lengths tab n' last_thing'
      return (ls ++ ws)

meta_code :: Word16 -> Code -> Length -> InfM ([Length], Word16, Length)
meta_code n i _ | i < 16 = let i' = fromIntegral i in return ([i'], n - 1, i')
meta_code n 16 last_thing
                 = do xs <- get_bits 2
                      let l = 3 + bits_to_word16 xs
                      return (genericReplicate l last_thing, n - l, last_thing)
meta_code n 17 _ = do xs <- get_bits 3
                      let l = 3 + bits_to_word16 xs
                      return (genericReplicate l 0, n - l, 0)
meta_code n 18 _ = do xs <- get_bits 7
                      let l = 11 + bits_to_word16 xs
                      return (genericReplicate l 0, n - l, 0)
meta_code _ i _ = error $ "meta_code: " ++ show i
\end{code}

\begin{code}
inflate_codes :: Bool -> Tables -> InfM Octets
inflate_codes seen_last tabs@(tab_litlen, tab_dist)
 =
   {- do done <- no_bits
      if done
        then return [] -- XXX Is this right?
        else -}
             do i <- tab_litlen;
                if i == 256
                  then inflate_blocks seen_last
                  else
                       do pref <- if i < 256
                                  then do let i' = fromIntegral i
                                          output i'
                                          return [i']
                                  else case lookup i litlens of
                                           Nothing -> error "do_code_litlen"
                                           Just (base, num_bits) ->
                                               do extra <- get_w16 num_bits
                                                  let l = base + extra
                                                  dist <- dist_code tab_dist
                                                  repeat_w32s l dist
                          o <- inflate_codes seen_last tabs
                          return (pref ++ o)
\end{code}

\begin{code}
litlens :: [(Code, (LitLen, Word8))]
litlens = zip [257..285] $ mk_bases 3 litlen_counts ++ [(258, 0)]
    where litlen_counts = [(8,0),(4,1),(4,2),(4,3),(4,4),(4,5)]

dist_code :: Table -> InfM Dist
dist_code tab
 = do code <- tab
      case lookup code dists of
          Nothing -> error "dist_code"
          Just (base, num_bits) -> do extra <- get_w16 num_bits
                                      return (base + extra)

dists :: [(Code, (Dist, Word8))]
dists = zip [0..29] $ mk_bases 1 dist_counts
    where dist_counts = (4,0):map ((,) 2) [1..13]

mk_bases :: Word16 -> [(Int, Word16)] -> [(Word16, Word8)]
mk_bases base counts = snd $ mapAccumL next_base base incs
            where next_base current bs = (current + 2^bs, (current, fromIntegral bs))
                  incs = concat $ map (uncurry replicate) counts
\end{code}

\section{Fixed tables}

The fixed tables. Not much to say really.

\begin{code}
inflate_trees_fixed :: Tables
inflate_trees_fixed = (make_table $ [(8, c) | c <- [0..143]]
                                 ++ [(9, c) | c <- [144..255]]
                                 ++ [(7, c) | c <- [256..279]]
                                 ++ [(8, c) | c <- [280..287]],
                       make_table [(5, c) | c <- [0..29]])
\end{code}

\section{The Huffman Tree}

As the name suggests, the obvious way to store Huffman trees is in a
tree datastructure. Externally we want to view them as functions though,
so we wrap the tree with \verb!get_code! which takes a list of bits and
returns the corresponding code and the remaining bits. To make a tree
from a list of length code pairs is a simple recursive process.

\begin{code}
data Tree = Branch Tree Tree | Leaf Code | Null

make_table :: [(Length, Code)] -> Table
make_table lcs = case make_tree 0 $ sort $ filter ((/= 0) . fst) lcs of
                     (tree, []) -> get_code tree
                     _ -> error $ "make_table: Left-over lcs from"

get_code :: Tree -> InfM Code
get_code (Branch zero_tree one_tree)
 = do Bit b <- get_bit
      if b then get_code one_tree else get_code zero_tree
get_code (Leaf w) = return w
get_code Null = error "get_code Null"

make_tree :: Length -> [(Length, Code)] -> (Tree, [(Length, Code)])
make_tree _ [] = (Null, [])
make_tree i lcs@((l, c):lcs')
 | i == l = (Leaf c, lcs')
 | i < l = let (zero_tree, lcs_z) = make_tree (i+1) lcs
               (one_tree, lcs_o) = make_tree (i+1) lcs_z
           in (Branch zero_tree one_tree, lcs_o)
 | otherwise = error "make_tree: can't happen"
\end{code}

